%Authors: 	Martin Holler and Kristian Bredies
%           	University of Graz
%Contact	martin.holler@uni-graz.at
%Usage permited only for academic purposes
%NOTE: This is NON-OPTIMIZED matlab code
%If you like and use this code, please cite:
%K. Bredies and M. Holler. A TGV regularized wavelet based zooming model. Scale Space and Variational Methods in Computer Vision, Springer LNCS, 7893:149-160, 2013.


%Wavelet based zooming function
%Input: Original image g, zooming factor z_exp (magnification by 2^z_exp), nr.
%of iterations it
%Output: Zoomed image g2

function [tgv_z,tgv,gap,data_error,max_data_error,sig_save,std,bil,cub,lanc2,lanc3,wave] = orth_wavelet_zoom(image_name,stopping_rule,stopping_parameter,z_exp,filter_name,alph0)

path('stand_func',path);
path('wavelet_complementary',path);

%Parameter##################################################################

%stopping_rule = 'iteration';
%stopping_parameter = 100;

sig = 1/3;
tau = 1/3;

%alph0=4;%sqrt(2.2)g
alph1=1;

p=2;
gamma = 1.001;

%Set wavelet filters
%filter_name = 'haar'; %'LeGall','CDF','haar'

%Set date penalty type
data_type = 'point';%'point','interval'
interval_size = 20;

%Boolean to activate TGV and Data-Error calculation
test = 1;
%Boolean to activate Image drawing during iterations
draw = 0;
draw_every = 5;

%Updata rate for extension of data variables
update_rate = 100;

%Initialization##############################################################

%Get image
g = imread(image_name);
g = double(g);
g = g-128;

%Set image dimensions and prepare data space
[n,m] = size(g);
zn = n*(2^z_exp);
zm = m*(2^z_exp);
data = zeros(zn,zm);
data(1:n,1:m) = g;
if (strcmp(filter_name,'haar'))
	data = data*(2^z_exp);
end
ldata = data(1:n,1:m); rdata = ldata;
if strcmp(data_type,'interval')
	ldata = ldata - interval_size*ones(n,m);
	rdata = rdata + interval_size*ones(n,m);
end



%TGV-zooming---------------------------------------------------------------
%Input: Low-resulution data data
%       Zooming factor = wavelet level: z_exp
%       Stopping paramter
%	Paramter
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------

%Initialization of TV-related variables------------------------------------

k = 0;     %iteration counter

%Storage variables
x=cat(3,zeros(zn),zeros(zn),zeros(zn)); %Primal Variable x=(u,w)in UxW

%set starting value to data
x(:,:,1) = wavetrans(data,-z_exp,'standard',filter_name);
%Extragradient
ext = x;
            
y = cat(3,zeros(zn),zeros(zn),zeros(zn),zeros(zn),zeros(zn)); %Set first Dual Variable y=(p,q)in WxV



%K = (G,A)'
%   G = (nabla - I , symgrad)'
%   A = wavecdf97



%Set standard methods------------------------------------------------------

%Straighforward magnification
std = uint8(round(imresize(g,2^(z_exp),'nearest'))+128);
%Lanczos 3 Interpolation
lanc3 = uint8(round(imresize(g,2^(z_exp),'lanczos3'))+128);
%Lanczos 2 Interpolation
lanc2 = uint8(round(imresize(g,2^(z_exp),'lanczos2'))+128);
%Bilinear Interpolation
bil = uint8(round(imresize(g,2^(z_exp),'bilinear'))+128);
%Cubic interpolation
cub = uint8(round(imresize(g,2^(z_exp),'cubic'))+128);
%Wavelet-based Interpolation
wave = uint8(round(x(:,:,1))+128);

%--------------------------------------------------------------------------


%Make new figure to display iterative update
if draw == 1
        figure(10)
        imagesc(x(:,:,1)); colormap gray(256); axis square; truesize;
        title(['TGV-based Zoom (It: ',num2str(k),')']);
end


%Initialize data variables
    if strcmp(stopping_rule,'gap')
        tgv=zeros(update_rate+1,1);
        gap=zeros(update_rate+1,1);
        data_error = zeros(update_rate+1,1);
        max_data_error = zeros(update_rate+1,1);
        l1_unb = zeros(update_rate+1,1);
        sig_save = zeros(update_rate+1,1);
        
        sig_save(1) = sig;
        [tgv(1),gap(1)] = get_gap_orth(x,y,data(1:n,1:m),z_exp,filter_name,p,gamma,alph1,alph0);
        
    elseif (test)
        tgv=zeros(stopping_parameter+1,1);
        gap = zeros(stopping_parameter+1,1);
        data_error = zeros(stopping_parameter+1,1);
        max_data_error = zeros(stopping_parameter+1,1);
        l1_unb = zeros(stopping_parameter+1,1);
        sig_save = zeros(stopping_parameter+1,1);
        
        sig_save(1) = sig;
        [tgv(1),gap(1)] = get_gap_orth(x,y,data(1:n,1:m),z_exp,filter_name,p,gamma,alph1,alph0);
    else
        tgv=0;
        gap=0;
        data_error=0;
        max_data_error=0;
        l1_unb = 0;
        sig_save = 0;        
    end

%Main loop for TGV decompression-------------------------------------------
%--------------------------------------------------------------------------


cont = true;
while cont
    
    
    %Algorithmic issues----------------------------------------------------
    
    %Set y+=y+sig*K*x{3}
    y=y+sig*(cat(3,  fgrad_1(ext(:,:,1))-ext(:,:,2:3),...
                     sym_bgrad_2(ext(:,:,2:3))));

    
    
    %Project y+=(p,q,r) to {(p,q,r):|p|<alph1,|q|<alph0,r = w_p_data(r)}                        
    y=cat(3,  y(:,:,1:2)./(cat(3,   (max(1,(sqrt((y(:,:,1)).^2+(y(:,:,2)).^2))./alph1)),...
                                    (max(1,(sqrt((y(:,:,1)).^2+(y(:,:,2)).^2))./alph1)))),...
              y(:,:,3:5)./(cat(3,   (max(1,(sqrt((y(:,:,3)).^2+(y(:,:,4)).^2+2*(y(:,:,5)).^2))./alph0)),...
                                    (max(1,(sqrt((y(:,:,3)).^2+(y(:,:,4)).^2+2*(y(:,:,5)).^2))./alph0)),...
                                    (max(1,(sqrt((y(:,:,3)).^2+(y(:,:,4)).^2+2*(y(:,:,5)).^2))./alph0)) )) );

    
    %Set x+=x-tau*K'(y+)    
    ext=x-tau*(cat(3,  -bdiv_1(y(:,:,1:2)) ,...
                       -y(:,:,1:2)-fdiv_2(y(:,:,3:5)) ));
    
            
            %Projekt to data
            tmp = wavetrans(ext(:,:,1),z_exp,'standard',filter_name);
            tmp(1:n,1:m) = data(1:n,1:m);
            ext(:,:,1) = wavetrans(tmp,-z_exp,'standard',filter_name);
            
            
    	[sig,tau] = get_adaptive_stepsize_orth(ext - x,sig,tau,z_exp,filter_name);  
	sig_save(k+2) = sig;

    
    
    	x=2*ext-x;  %calculate extragradient
	
	[x,ext] = deal(ext,x); %Swap extragradient and primal variable
    
    k = k+1; %Update counter
    %----------------------------------------------------------------------
    
    
    %Data evaluation-------------------------------------------------------
    
    if rem(k,10) == 0
        display(['Finished iteration ',num2str(k)])
    end
    
    %Draw image    
    if ( draw == 1 && rem(k,draw_every) == 0 )
            figure (10)
            imagesc(x(:,:,1)); colormap gray(256); axis square;
            title(['TGV-based Zoom (It: ',num2str(k),')']);
            drawnow
    end
    

    %Evaluate stopping rule     
    if ( strcmp(stopping_rule,'gap') || test)

                %Set new gap
                [tgv(k+1),gap(k+1)] = get_gap_orth(x,y,data(1:n,1:m),z_exp,filter_name,p,gamma,alph1,alph0);
                
                %Get average and maximal data error
                err = wavetrans(x(:,:,1),z_exp,'standard',filter_name) - data;
                data_error(k+1) = sum(sum(abs( err(1:n,1:m) )))./(n*m);%!!!Data error only for known data!!!
                max_data_error(k+1) = max(max(abs( err(1:n,1:m) )));
                
                %Test stopping rule
                if strcmp(stopping_rule,'gap')
                    if (gap(k+1)<stopping_parameter && k>=50)
                        cont = false;
                    end
                end

    end
    
    		%Iteration:
    		if strcmp(stopping_rule,'iteration')
             		if k>=stopping_parameter
                    		cont = false;
             		end
    		end
     
end%----------------------------------------------------------------------
	
	%Crop sig_save
	if ( strcmp(stopping_rule,'gap') || test )
		sig_save = sig_save(1:k);
	end
	
	%Crop down tgv and gap
    	if ( strcmp(stopping_rule,'gap') );
        	tgv = tgv(1:k+1);
        	gap = gap(1:k+1);
       		data_error = data_error(1:k+1);
        	max_data_error = max_data_error(1:k+1);
    	end

%End of main loop for TGV decompression-------------------------------------
%--------------------------------------------------------------------------


%Set output
tgv_z=uint8(round(x(:,:,1)+128)); %set g2 to TGV decompression


%Display Results-----------------------------------------------------------
close all

%Show Standard, Lanczos 3 ; Wavelet, TGV
imshowTruesize({std,lanc3;wave,tgv_z});
display(['Fig. 1 Alignment:  Standard Lanczos3 '...
       ; '                   Wavelet  TGV-based']);
display(['------------------------------------']);
%Show Bilinear, Cubic ; Lanczos2, TGV
imshowTruesize({bil,cub;lanc2,tgv_z});
display(['Fig. 2 Alignment:  Bilinear Cubic    '...
       ; '                   Lanczos2 TGV-based']);
display(['Nr. of iterations: ',num2str(k)]);

if test == 1

    %Set stepsize for data points
    stepsize = ceil(k/update_rate);

    %Display TGV development
    figure('Name','TGV Development','NumberTitle','off');
    plot(tgv(1:stepsize:end));

    %Display primal dual gap
    figure('Name','Primal dual Gap','NumberTitle','off');
    plot(gap(1:stepsize:end));
    
    %Display maximal error
    %figure('Name','Maximal error','NumberTitle','off');
    %plot(max_data_error(1:stepsize:end));
    
    %Display average error
    %figure('Name','Average error','NumberTitle','off');
    %plot(data_error(1:stepsize:end));
end

    %If no TGV information is available, return number of iterations in tgv
    if ( (test == 0) && strcmp(stopping_rule,'iteration') )
    	tgv = k;
    end

%--------------------------------------------------------------------------
