


function [sig,tau,nksq] = set_opt_par(nwsq)

%We want to bound the norm of the global operator by above. As on sheet
%JPEG2000 bound, we know that |k(u,v)|^2 \leq (8+|W|^2) |u|^2 +
%2sqrt(8)|u||v| + 9|v|^2, where |W| is the norm of the W-transform operator

%Input of this function is thus the norm^2 of the W-transform operator.
%Output is are the possible stepsizes sig, tau and the norm^2 of the
%operator K


%We want to "hide" 2 sqrt(8) in 8+|W|^2 and 9, resp, using that, for a,b \in
%\R, 2ab \leq a^2 + b^2. So our first constraint is ab = sqrt(8). Since we
%want to have the coefficients of |u|^2 and |v|^2 to be equal, to achieve
%an optimal bound, the second constraint on a,b is 8+|W|^2 + a^2 = 9 + b^2 ->
%b^2 = a^2 -1 + |W|^2

c = nwsq;

%Solving this system we get that

a = sqrt( ( 1-c )/2 + sqrt( ( ( 1-c )^2 )/4 + 8));

%Test
%nk = sqrt( 8 + c + a^2);
%b = sqrt( (c-1)/2 + sqrt( ((c-1)/2)^2 + 8 ) );
%nk2 = sqrt( 9 + b^2);

b = sqrt( a^2 - 1 + c );

%Thus we get that our norm bound^2 equals 8+c^n + a^2 = 9 + b^2
%Test calculation:
if ( 8+c + a^2 - 9 - b^2 >= 10^(-10))
    display('!!!Stepsize error!!!');
    display(['Difference: ',num2str(8+c+a^2 - 9 - b^2)]);
end

nksq = 9+b^2;

sig = sqrt(1/(9 + b^2));
tau = sqrt(1/(9 + b^2));
